@@ -32,6 +32,7 @@
 #include <limits>
 #include <string>
 #include "ceres/internal/eigen.h"
+#include "ceres/is_close.h"
 #include "ceres/internal/port.h"
 #include "ceres/jet.h"
 #include "ceres/rotation.h"
@@ -616,9 +617,10 @@ static void Transpose3x3(double m[9]) {
 }
 
 // Convert Euler angles from radians to degrees.
-static void ToDegrees(double ea[3]) {
-  for (int i = 0; i < 3; ++i)
-    ea[i] *= 180.0 / kPi;
+static void ToDegrees(double euler_angles[3]) {
+  for (int i = 0; i < 3; ++i) {
+    euler_angles[i] *= 180.0 / kPi;
+  }
 }
 
 // Compare the 3x3 rotation matrices produced by the axis-angle
@@ -662,13 +664,13 @@ TEST(EulerAnglesToRotationMatrix, OnAxis) {
 TEST(EulerAnglesToRotationMatrix, IsOrthonormal) {
   srand(5);
   for (int trial = 0; trial < kNumTrials; ++trial) {
-    double ea[3];
-    for (int i = 0; i < 3; ++i)
-      ea[i] = 360.0 * (RandDouble() * 2.0 - 1.0);
-    double ea_matrix[9];
-    ToDegrees(ea);  // Radians to degrees.
-    EulerAnglesToRotationMatrix(ea, 3, ea_matrix);
-    EXPECT_THAT(ea_matrix, IsOrthonormal());
+    double euler_angles_degrees[3];
+    for (int i = 0; i < 3; ++i) {
+      euler_angles_degrees[i] = RandDouble() * 360.0 - 180.0;
+    }
+    double rotation_matrix[9];
+    EulerAnglesToRotationMatrix(euler_angles_degrees, 3, rotation_matrix);
+    EXPECT_THAT(rotation_matrix, IsOrthonormal());
   }
 }
 
@@ -697,25 +699,20 @@ J4 MakeJ4(double a, double v0, double v1, double v2, double v3) {
   return j;
 }
 
-
 bool IsClose(double x, double y) {
   EXPECT_FALSE(IsNaN(x));
   EXPECT_FALSE(IsNaN(y));
-  double absdiff = fabs(x - y);
-  if (x == 0 || y == 0) {
-    return absdiff <= kTolerance;
-  }
-  double reldiff = absdiff / max(fabs(x), fabs(y));
-  return reldiff <= kTolerance;
+  return internal::IsClose(x, y, kTolerance, NULL, NULL);
 }
 
 template <int N>
 bool IsClose(const Jet<double, N> &x, const Jet<double, N> &y) {
-  if (IsClose(x.a, y.a)) {
-    for (int i = 0; i < N; i++) {
-      if (!IsClose(x.v[i], y.v[i])) {
-        return false;
-      }
+  if (!IsClose(x.a, y.a)) {
+    return false;
+  }
+  for (int i = 0; i < N; i++) {
+    if (!IsClose(x.v[i], y.v[i])) {
+      return false;
     }
   }
   return true;
@@ -814,9 +811,9 @@ TEST(Rotation, SmallQuaternionToAngleAxisForJets) {
     J4 quaternion[4] = { J4(c, 0), J4(s, 1), J4(0, 2), J4(0, 3) };
     J4 axis_angle[3];
     J4 expected[3] = {
-        MakeJ4(s, -2*theta, 2*theta*c, 0, 0),
-        MakeJ4(0, 0, 0, 2*theta/s, 0),
-        MakeJ4(0, 0, 0, 0, 2*theta/s),
+        MakeJ4(2*theta, -2*s, 2*c,  0,         0),
+        MakeJ4(0,        0,   0,    2*theta/s, 0),
+        MakeJ4(0,        0,   0,    0,         2*theta/s),
     };
     QuaternionToAngleAxis(quaternion, axis_angle);
     ExpectJetArraysClose<3, 4>(axis_angle, expected);
@@ -837,9 +834,9 @@ TEST(Rotation, TinyQuaternionToAngleAxisForJets) {
     // a finite expansion is used here, which will
     // be exact up to machine precision for the test values used.
     J4 expected[3] = {
-        MakeJ4(theta, -2*theta, 2.0, 0, 0),
-        MakeJ4(0, 0, 0, 2.0, 0),
-        MakeJ4(0, 0, 0, 0, 2.0),
+        MakeJ4(2*theta, -2*s, 2.0, 0,   0),
+        MakeJ4(0,        0,   0,   2.0, 0),
+        MakeJ4(0,        0,   0,   0,   2.0),
     };
     QuaternionToAngleAxis(quaternion, axis_angle);
     ExpectJetArraysClose<3, 4>(axis_angle, expected);
