@@ -35,6 +35,7 @@
 
 from __future__ import print_function
 import glob
+import io
 import os
 import sys
 
@@ -75,7 +76,7 @@ replacements = [
 
   # The title for the homepage is not ideal, so change it.
   ('<title>Ceres Solver &mdash; Ceres Solver</title>',
-   '<title>Ceres Solver &mdash; A Nonlinear Least Squares Minimizer</title>')
+   '<title>Ceres Solver &mdash; A Large Scale Non-linear Optimization Library</title>')
 ]
 
 # This is a nasty hack to strip the breadcrumb navigation. A better strategy is
@@ -85,7 +86,7 @@ breadcrumb_start_other = \
 '''<div role="navigation" aria-label="breadcrumbs navigation">
   <ul class="wy-breadcrumbs">
     <li><a href="index.html">Docs</a> &raquo;</li>
-      
+
     <li>'''
 
 # The index page has a slightly different breadcrumb.
@@ -94,7 +95,7 @@ breadcrumb_start_index = breadcrumb_start_other.replace('index.html', '#')
 breadcrumb_end = \
 '''</li>
       <li class="wy-breadcrumbs-aside">
-        
+
       </li>
   </ul>
   <hr/>
@@ -102,7 +103,7 @@ breadcrumb_end = \
 
 for name in glob.glob('%s/*.html' % html_dir):
   print('Postprocessing: ', name)
-  with open(name) as fptr:
+  with io.open(name, encoding="utf-8") as fptr:
     out = fptr.read()
 
   for input_pattern, output_pattern in replacements:
@@ -119,5 +120,5 @@ for name in glob.glob('%s/*.html' % html_dir):
   except ValueError:
     print('Skipping breadcrumb strip for', name)
 
-  with open(name, 'w') as fptr:
+  with io.open(name, 'w', encoding="utf-8") as fptr:
     fptr.write(out)
